from gi.repository import Gtk,GObject

from time import sleep

from enum import IntEnum
class COLUMNS(IntEnum):
	NAME=0
	SIZE=1
	TTH=2
listcols="str,GObject.TYPE_INT64,str"

from . import reqs
from . import hubs
from . import overrides

name=Gtk.Label()
folder=Gtk.Label()
bx=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
list=eval("Gtk.ListStore("+listcols+")")
sort=Gtk.TreeModelSort.new_with_model(list)
sep="\\"

def cols(tree,act):
	hubs.col(tree,'Name',COLUMNS.NAME,act)
	hubs.col(tree,'Size',COLUMNS.SIZE,act)
	hubs.col(tree,'TTH',COLUMNS.TTH,act)
def show():
	scroll=Gtk.ScrolledWindow()
	scroll.set_vexpand(True)
	tree=hubs.TreeView(sort)
	cols(tree,clk)
	tree.connect("row-activated",clkrow,sort)
	tree.set_activate_on_single_click(True)
	scroll.set_child(tree)
	bx.append(name)
	bx.append(folder)
	back=Gtk.Button.new_with_label('..')
	back.connect('clicked',backing,None)
	bx.append(back)
	bx.append(scroll)
	return bx
def clk(b,ix):
	hubs.clk_univ(sort,ix)
def clkrow(tree,path,column,model):
	it=model.get_iter(path)
	fpath=folder.get_text()+model.get_value(it,COLUMNS.NAME)
	if not model.get_value(it,COLUMNS.TTH):
		fshow(name.get_text(),fpath+sep)
	else:
		reqs.requ("list.downloadfile",{"downloadto" : "","filelist" : name.get_text(),"target" : fpath})

def set(nb,nm):
	name.set_text(nm)
	z=nm.split(".")
	nb.set_tab_label_text(bx,z[0])
	reqs.req("list.closeall")
	reqs.requ("list.open",{"filelist" : nm})
	fshow(nm,'')

def backing(b,d):
	s=folder.get_text()
	if not s:
		return
	p=s.rfind(sep,0,-len(sep))
	if p!=-1:
		fshow(name.get_text(),s[:p+len(sep)])
	else:
		fshow(name.get_text(),'')
def fshow(flist,s):
	#wait phisical read,listopened was not a solution
	for i in range(0,5):
		a=reqs.reque("list.lsdir",{"directory" : s,"filelist" : flist})
		if a:
			folder.set_text(s)
			list.clear()
			for x in a:
				e=a[x]
				if "TTH" in e:
					t=e['TTH']
				else:
					t=''
				overrides.append(list,[x,int(e['Size']),t])
			break
		else:
			sleep(1)



import requests
import random

from . import com

url = "http://localhost:3121"
jsonrpcversion="2.0"

def req(a):
	payload = {
		"id" : random.randint(0,(2**16)-1),
		"jsonrpc" : jsonrpcversion,
		"method" : a,
		"version" : jsonrpcversion
	}
	return incom(payload)
def requ(a,para):
	payload = {
		"id" : random.randint(0,(2**16)-1),
		"jsonrpc" : jsonrpcversion,
		"method" : a,
		"version" : jsonrpcversion,
		"params" : para
	}
	incom(payload)
def reque(a,para):
	payload = {
		"id" : random.randint(0,(2**16)-1),
		"jsonrpc" : jsonrpcversion,
		"method" : a,
		"version" : jsonrpcversion,
		"params" : para
	}
	return incom(payload)

def incom(payload):
	response = requests.post(url, json=payload).json()
	com.post(response)
	return response['result']

def reque_simple(a,para):
	payload = {
		"id" : random.randint(0,(2**16)-1),
		"jsonrpc" : jsonrpcversion,
		"method" : a,
		"version" : jsonrpcversion,
		"params" : para
	}
	return requests.post(url, json=payload).json()


from . import users
from . import hubs
from . import search
from . import overrides

class detail():
	def __init__(self,a,b,c,d):
		self.nick=a
		self.hub=b
		self.fslots=c
		self.fname=d

def create(r,free):
	return detail(r["Nick"],r["Hub URL"],free,r[search.ext_fn])

def update(r,free,lst,it,col):
	ar=lst.get_value(it,col)
	ar.append(create(r,free))
	lst.set_value(it,col,ar)

from gi.repository import Gtk

from enum import IntEnum
class COLUMNS(IntEnum):
	NICK=0
	HUB=1
	FSLOTS=2
	FNAME=3

list=Gtk.ListStore(str,str,str,str)
sort=Gtk.TreeModelSort.new_with_model(list)

def show():
	scroll=Gtk.ScrolledWindow()
	scroll.set_vexpand(True)
	tree=hubs.TreeView(sort)
	hubs.col(tree,'Nick',COLUMNS.NICK,clk)
	hubs.col(tree,'Hub URL',COLUMNS.HUB,clk)
	hubs.col(tree,'Free Slots',COLUMNS.FSLOTS,clk)
	hubs.col(tree,'Name',COLUMNS.FNAME,clk)
	tree.connect("row-activated",clkrow,sort)
	tree.set_activate_on_single_click(True)
	scroll.set_child(tree)
	bx=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	b=Gtk.Button.new_with_label(chr(0x1F50D))
	b.connect('clicked', srch, tree)
	bx.append(scroll)
	bx.append(b)
	return bx
def clk(b,ix):
	hubs.clk_univ(sort,ix)
def clkrow(tree,path,column,model):
	it=model.get_iter(path)
	users.ldload(model.get_value(it,COLUMNS.HUB),model.get_value(it,COLUMNS.NICK))

def set(ar):
	list.clear()
	for x in ar:
		overrides.append(list,[x.nick,x.hub,x.fslots,x.fname])

def srch(b,tree):
	s=tree.get_selection()
	d=s.get_selected()#iter free is in the bindings
	if d[1]:
		search.start(d[0].get_value(d[1],COLUMNS.FNAME))



from gi.repository import Gtk

from . import reqs
from . import users
from . import flist
from . import overrides

list=users.listdef()
page=Gtk.ScrolledWindow()

def show(nb):
	page.set_vexpand(True)
	sort=Gtk.TreeModelSort.new_with_model(list)
	users.show_univ(nb,page,sort,clkrow)
	return page

def clkrow(t,p,c,b):
	m=t.get_model()
	user=m.get_value(m.get_iter(p),0)
	flist.set(b,user)

def set():
	s=";"
	r=reqs.reque("list.local",{"separator" : s})
	list.clear()
	if r:
		usrs=r.split(s)
		for x in usrs:
			overrides.append(list,[x])


from gi.repository import Gtk,GLib

from . import reqs
from . import flist
from . import hubs
from . import search
from . import overrides

from enum import IntEnum
class COLUMNS(IntEnum):
	NAME=0
	PATH=1
	DOWNLOADED=2
	SIZE=3
	USERS=4
	STATUS=5

list=Gtk.ListStore(str,str,str,int,str,str)
sort=Gtk.TreeModelSort.new_with_model(list)
page=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
timer=0

def show():
	scroll=Gtk.ScrolledWindow()
	scroll.set_vexpand(True)
	tree=hubs.TreeView(sort)
	hubs.col(tree,'Name',COLUMNS.NAME,clk)
	hubs.col(tree,'Path',COLUMNS.PATH,clk)
	hubs.col(tree,'Downloaded',COLUMNS.DOWNLOADED,clk)
	hubs.col(tree,'Size',COLUMNS.SIZE,clk)
	hubs.col(tree,'Users',COLUMNS.USERS,clk)
	hubs.col(tree,'Status',COLUMNS.STATUS,clk)
	tree.connect("row-activated",clkrow,sort)
	tree.set_activate_on_single_click(True)
	scroll.set_child(tree)
	b=Gtk.Button.new_with_label("-")
	b.connect('clicked', rem, tree)
	page.append(scroll)
	page.append(b)
	return page
def clk(b,ix):
	hubs.clk_univ(sort,ix)
def clkrow(tree,path,column,model):
	it=model.get_iter(path)
	search.start(model.get_value(it,COLUMNS.NAME))

def add(m,it):
	xt=m.get_value(it,flist.COLUMNS.TTH)
	xl=m.get_value(it,flist.COLUMNS.SIZE)
	dn=m.get_value(it,flist.COLUMNS.NAME)
	m="magnet:?xt=urn:tree:tiger:"+xt+"&xl="+str(xl)+"&dn="+dn
	reqs.requ("magnet.add",{"directory" : "","magnet" : m})

def set():
	global timer
	if timer==0:
		if fresh(None):
			timer=GLib.timeout_add_seconds(10,fresh,None)
def fresh(d):
	rows=reqs.req("queue.list")
	for x in list:
		if inspect(x.iter,rows):
			list.remove(x.iter)
	if rows:
		for t in rows:
			r=rows[t]
			overrides.append(list,[r["Filename"],r["Path"],r["Downloaded"],int(r["Size Sort"]),r["Users"],r["Status"]])
		return True
	if len(list)==0:
		global timer
		timer=0
		return False
	return True
def inspect(it,rows):
	if not rows:
		return True
	told=target(list,it)
	for t in rows:
		r=rows[t]
		tnew=r["Path"]+r["Filename"]
		if told==tnew:
			list.set_value(it,COLUMNS.DOWNLOADED,r["Downloaded"])
			list.set_value(it,COLUMNS.SIZE,int(r["Size Sort"]))
			list.set_value(it,COLUMNS.USERS,r["Users"])
			list.set_value(it,COLUMNS.STATUS,r["Status"])
			del rows[t]
			return False
	return True
def close():
	if timer:
		GLib.source_remove(timer)

def target(model,iter):
	return model.get_value(iter,COLUMNS.PATH)+model.get_value(iter,COLUMNS.NAME)
def rem(b,t):
	s=t.get_selection()
	d=s.get_selected()#iter free is in the bindings
	if d[1]:#on tab focus is selecting but on force click is ugly
		tg=target(d[0],d[1])
		reqs.requ("queue.remove",{"target":tg})
		list.remove(d[0].convert_iter_to_child_iter(d[1]))




from gi.repository import GLib

import time

from . import hubs
from . import reqs
from . import hubson
from . import users

recons=[]
class reconnect():
	def __init__(self,adr):
		self.adr=adr

def acon(a):
	try:
		reqs.requ("hub.add",{"enc" : "", "huburl" : a})
	except Exception:
		r=reconnect(a)
		r.id=GLib.timeout_add_seconds(3,bcon,r)
		recons.append(r)
def bcon(r):
	acon(r.adr)
	recons.remove(r)
	return False

def add(tree,path,column,model):
	it=model.get_iter(path)#iter free is in the bindings
	d=[]
	for i in hubs.COLUMNS:
		d.append(model.get_value(it,i))
	addcon(model,d)
def addcon(model,rowlst):
	adr=rowlst[hubs.COLUMNS.ADDRESS]
	lst=hubson.list
	for z in lst:
		x=lst.get_value(z.iter,hubs.COLUMNS.ADDRESS)
		if x==adr:
			return
	acon(adr)
	hubson.add(rowlst)
def remcon(nb,a):
	reqs.requ("hub.del",{"huburl" : a})
	hclose(a)
	users.ifclear(nb,a)

def recon():
	lst=hubson.list
	for z in lst:
		x=lst.get_value(z.iter,hubs.COLUMNS.ADDRESS)
		for a in recons:
			if x==a.adr:
				return
		acon(x)

def close():
	for x in recons:
		GLib.source_remove(x.id)
def hclose(a):
	for x in recons:
		if x.adr==a:
			GLib.source_remove(x.id)
			return




from . import limit
from . import log
from . import stor2
from . import nick
from . import hubs
from . import hubson
from . import search
from . import daem
from . import share
from . import com

import json
from gi.repository import Gdk

import appdirs
import os.path
import pathlib
def get_client_dir():
	return pathlib.Path(appdirs.user_config_dir('dicopp'))
get_client_dir().mkdir(exist_ok=True)
def get_client():
	return os.path.join(get_client_dir(),'config.json')

def write(win):
	d={}
	dim=win.get_default_size()
	d['width']=dim.width
	d['height']=dim.height
	d['max']=win.is_maximized()
	d['min']=win.get_surface().get_state()&Gdk.ToplevelState.MINIMIZED
	limit.store(d)
	log.store(d)
	stor2.store(d)
	nick.store(d)
	hubs.store(d)
	hubson.store(d)
	search.store(d)
	daem.store(d)
	share.store(d)
	com.store(d)
	with open(get_client(), "w") as write_file:
		json.dump(d, write_file)

def read(win):
	try:
		with open(get_client()) as f:
			d=json.load(f)
			win.set_default_size(d['width'],d['height'])
			if(d['max']):
				win.maximize()
			if(d['min']):
				win.minimize()
			limit.restore(d)
			log.restore(d)
			stor2.restore(d)
			nick.restore(d)
			hubs.restore(d)
			search.restore(d)
			daem.restore(d)
			share.restore(d)
			com.restore(d)
			return d
	except Exception:
		return None
def read2(d):
	if d:
		hubson.restore(d)



from gi.repository import GLib,Gtk

from . import reqs
from . import log
from . import sets

start=0
time=Gtk.EntryBuffer(text="0")
limit=Gtk.EntryBuffer(text='1000000000')
timer=0
def timer_get():
	return int(time.get_text())

def open(win):
	t=timer_get()
	if t>0:
		global timer
		timer=GLib.timeout_add_seconds(t*60,callba,win)

def close():
	global timer
	if timer:
		GLib.source_remove(timer)
		timer=0

def callba(win):
	res=reqs.req("show.ratio")
	upB=int(res['up_bytes'])-start
	log.add(upB)
	if upB>int(limit.get_text()):
		print("Upload limit. Window close.")
		global timer
		timer=0
		win.close()
		return False
	return True

def confs():
	f=Gtk.Frame()
	f.set_label("Close program when upload is greater than Value in bytes")
	g=Gtk.Grid()
	lb=Gtk.Label()
	lb.set_halign(Gtk.Align.START)
	lb.set_text("Interval time to verify in minutes (0=disable)")
	g.attach(lb,0,0,1,1)
	en=sets.entries(time)
	g.attach(en,1,0,1,1)
	lb=Gtk.Label()
	lb.set_halign(Gtk.Align.START)
	lb.set_text("Value")
	g.attach(lb,0,1,1,1)
	en=sets.entries(limit)
	g.attach(en,1,1,1,1)
	f.set_child(g)
	return f
def reset(w):
	close()
	open(w)

def store(d):
	d['upload_time']=int(time.get_text())
	d['upload_limit']=int(limit.get_text())

def restore(d):
	time.set_text(str(d['upload_time']),-1)
	limit.set_text(str(d['upload_limit']),-1)


from gi.repository import Gtk

import random
import xml.etree.ElementTree as ET

from . import stor2
from . import daem
from . import share

def rand4(a):
	s=str(a)
	while len(s)<4:
		s='0'+s
	return s
name=Gtk.EntryBuffer(text='dico_'+rand4(random.randint(0,9999)))

def confs():
	return name
def store(d):
	d['nick_name']=name.get_text()
def restore(d):
	name.set_text(d['nick_name'],-1)
def ini(restart):
	f=stor2.get_file()
	t = ET.parse(f)
	root = t.getroot()
	se = root.find(stor2.set)
	b=see(se,'Nick','string',name)
	c=share.ini(root) or b
	#Slots not working r|=see(se,'Slots','int',slots)
	if c:
		if restart:
			daem.dclose()
		t.write(f)
		if restart:
			daem.restart()
	return c

def see(se,n,t,b):
	txt=b.get_text()
	s = se.find(n)
	if s==None:
		s=ET.SubElement(se,n)
		s.set('type',t)
		s.text=txt
		return True
	elif txt!=s.text:
		s.text=txt
		return True
	return False



import gi
gi.require_version("Gtk", "4.0")
from gi.repository import Gtk,GLib

from . import base
from . import layout 
from . import limit
from . import log
from . import stor2
from . import nick
from . import hubs
from . import hubscon
from . import daem
from . import search
from . import dload
from . import com

def quit(widget, mainloop):
	base.write(widget)
	daem.close(False)#in base.write is log, can require daemon open
	limit.close()
	hubscon.close()
	search.close()
	dload.close()
	com.close()
	mainloop.quit()
	return True

def main():
	mainloop = GLib.MainLoop()
	win = Gtk.Window()
	win.set_title('Direct Connect')
	d=base.read(win)
	layout.show(win)
	limit.open(win)
	log.ini()
	stor2.ini()
	nick.ini(False)
	hubs.ini()
	win.connect('close-request', quit, mainloop)
	daem.dopen()
	base.read2(d)#after daemon start
	win.show()
	mainloop.run()

if __name__ == "__main__":
    main()




from gi.repository import Gtk
import xml.etree.ElementTree as ET

from . import sets

folder=Gtk.EntryBuffer()#text=''

def confs():
	return sets.entry("Share folder",folder)
def store(d):
	d['share_folder']=folder.get_text()
def restore(d):
	folder.set_text(d['share_folder'],-1)
#eiskalt want only with trailing /
#this method at reread will work
#with share.add will not work without /
def ini(root):
	se = root.find("Share")
	dr="Directory"
	s = se.find(dr)
	d=folder.get_text()
	if d:
		if s==None:
			s=ET.SubElement(se,dr)
			s.set('Virtual','share')
			s.text=d
			return True
		elif d!=s.text:
			s.text=d
			return True
	elif s!=None:
		se.clear()
		return True
	return False




from gi.repository import Gtk

from . import sets
from . import hubs
from . import hubson
from . import users
from . import usersloc
from . import flist
from . import search
from . import details
from . import dload
from . import com

def show(w):
	bx=Gtk.Box()
	bx.set_orientation(Gtk.Orientation.VERTICAL)
	b=Gtk.Button.new_with_label(chr(0x2699))
	b.connect('clicked', sets.ini, w)
	box=Gtk.Box()
	box.append(b)
	pags=Gtk.Notebook()#scrollable=True is Bus error here,use Tab to select and left right
	e=Gtk.Entry(hexpand=True)
	e.set_placeholder_text('Search...')
	e.connect('activate',search.send,pags)
	box.append(e)
	bx.append(box)
	pags.append_page(hubs.show(),Gtk.Label(label="HubList"))
	pags.append_page(hubson.show(pags),Gtk.Label(label="Hubs"))
	pags.append_page(users.show(pags,w),Gtk.Label(label=users.intro))
	pags.append_page(usersloc.show(pags),Gtk.Label(label="Users"))
	pags.append_page(flist.show(),Gtk.Label(label="FileList"))
	pags.append_page(search.show(),Gtk.Label(label="Search"))
	pags.append_page(details.show(),Gtk.Label(label="Details"))
	pags.append_page(dload.show(),Gtk.Label(label="Downloads"))
	pags.append_page(com.show(),Gtk.Label(label="Command"))
	pags.connect("switch-page",sw,None)
	bx.append(pags)
	w.set_child(bx)

def sw(notebook,page,page_num,d):
	if page==hubson.page:
		hubson.set()
	elif page==usersloc.page:
		usersloc.set()
	elif page==search.page:
		search.set()
	elif page==dload.page:
		dload.set()



from gi.repository import Gtk

from . import sets,reqs

def ini(hub,user,win):
	r=reqs.reque("hub.getuserinfo",{"nick":user,"huburl":hub})
	if r:
		d=sets.dial(user,win,done,None)
		d.add_button("_OK",Gtk.ResponseType.NONE)
		bx=Gtk.Grid()
		bx.set_column_spacing(5)
		y=0
		for k in r:
			bx.attach(Gtk.Label(label=k,halign=Gtk.Align.START),0,y,1,1)
			y+=1
		y=0
		for v in r.values():
			bx.attach(Gtk.Label(label=v,halign=Gtk.Align.START),1,y,1,1)
			y+=1
		box=d.get_content_area()
		box.append(Gtk.ScrolledWindow(child=bx,hexpand=True,vexpand=True))
		d.show()

def done(w,r,d):
	#if r==Gtk.ResponseType.NONE:
	w.destroy()#ok with or without at X button




def append(list,values):
	values,columns=list._convert_row(values)
	try:
		list.insert_with_values(-1,columns,values)
	except AttributeError:#support gtk-4.0
		list.insert_with_valuesv(-1,columns,values)




from gi.repository import Gtk

from . import limit
from . import log
from . import stor2
from . import nick
from . import hubs
from . import search
from . import daem
from . import share
from . import com

class entries(Gtk.Entry):
	def __init__(self,bf):
		Gtk.Entry.__init__(self,buffer=bf,hexpand=True)
		#when write quick, hexpand only at first
def dial(t,win,f,data):
	d=Gtk.Dialog(title=t,transient_for=win)
	d.set_modal(True)
	d.connect("response",f,data)
	if win.is_maximized():
		d.maximize()
	else:
		dim=win.get_default_size()
		d.set_default_size(dim.width,dim.height)
	return d
def ini(b,win):
	d=dial("Settings",win,reset,win)
	d.add_button("_OK",Gtk.ResponseType.NONE)
	box=d.get_content_area()
	bx=Gtk.Box()
	bx.set_orientation(Gtk.Orientation.VERTICAL)
	bx.append(limit.confs())
	bx.append(log.confs())
	bx.append(stor2.confs())
	bx.append(hubs.confs())
	bx.append(search.confs(d))
	bx.append(daem.confs())
	bx.append(share.confs())
	bx.append(com.confs())
	box.append(Gtk.ScrolledWindow(child=bx,vexpand=True))
	d.show()
def reset(d,r,w):
	limit.reset(w)
	log.reset()
	stor2.ini()
	wasreset=nick.ini(True)
	hubs.reset()
	search.reset()
	if not wasreset:
		daem.reset()
	d.destroy()#at X can be omitted

def entry(txt,buf):
	bx=Gtk.Box()
	lb=Gtk.Label()
	lb.set_halign(Gtk.Align.START)
	lb.set_text(txt)
	bx.append(lb)
	en=Gtk.Entry.new_with_buffer(buf)
	en.set_hexpand(True)
	bx.append(en)
	return bx



from gi.repository import Gtk

from . import search
from . import sets
from . import overrides

list=Gtk.ListStore(str)

def store(d):
	add()
	a=[]
	for x in list:
		a.append(list.get_value(x.iter,0))
	d['search_extensions']=a
def restore(d):
	for x in d['search_extensions']:
		ad([x])
	search.extensions.set_text(d['search_extensions'][0],-1)
def confs(en,win):
	bx=Gtk.Box()
	b=Gtk.Button.new_with_label("Change")
	b.connect('clicked', sel, win)
	bx.append(en)
	bx.append(b)
	return bx

def ad(x):
	overrides.append(list,x)
def add():
	c=search.extensions.get_text()
	for x in list:
		if c==list.get_value(x.iter,0):
			return
	ad([c])

def sel(b,win):
	add()
	tree=Gtk.TreeView.new_with_model(list)
	renderer = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn()
	column.pack_start(renderer,True)
	column.add_attribute(renderer, "text", 0)
	tree.append_column(column)
	d=sets.dial("Extensions",win,reset,tree)
	d.add_button("_Set",Gtk.ResponseType.YES)
	d.add_button("_Delete",Gtk.ResponseType.NO)
	scroll=Gtk.ScrolledWindow(hexpand=True,vexpand=True)
	scroll.set_child(tree)
	d.get_content_area().append(scroll)
	d.show()

def reset(d,r,t):
	if r!=Gtk.ResponseType.DELETE_EVENT:
		x=t.get_selection()
		s=x.get_selected()
		if s[1]:
			if r==Gtk.ResponseType.YES:
				s[0].move_after(s[1],None)
				search.extensions.set_text(s[0].get_value(s[1],0),-1)
				d.destroy()
			elif r==Gtk.ResponseType.NO:
				s[0].remove(s[1])



from gi.repository import Gtk
import subprocess
import shlex
import psutil

data=Gtk.EntryBuffer()#text=''

from . import sets
from . import hubscon
from . import reqs

name='eiskaltdcpp-daemon'
instobj=None

def confs():
	global keep
	keep=data.get_text()
	return sets.entry("Daemon parameters",data)
def store(d):
	d['daemon_args']=data.get_text()
def restore(d):
	data.set_text(d['daemon_args'],-1)
def reset():
	if keep!=data.get_text():
		dclose()
		restart()

def restart():
	open()
	hubscon.recon()

def runs():
	for proc in psutil.process_iter():
		try:#/y/x proc.name() = x
			if name==proc.name():
				return proc
		except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
			pass
	return None
def dopen():
	global absent
	absent=runs()==None
	if absent:
		open()
def dclose():
	close(True)
def open():
	seq=shlex.split(data.get_text())
	args=[name]
	for s in seq:
		args.append(s)
	global instobj
	instobj=subprocess.Popen(args)#otherwise, cannot make it works
def close(inter):
	if absent:
		instobj.terminate()
		instobj.wait()
	elif inter:
		if instobj:
			instobj.terminate()
			instobj.wait()
		else:
			p=psutil.Process(runs().pid)
			reqs.req("daemon.stop")
			p.wait()


from gi.repository import Gtk

from . import hubs
from . import reqs
from . import overrides
from . import userinfo

listdef=lambda:Gtk.ListStore(str)

list=listdef()
page=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
intro="UserList"

def show(nb,win):
	scroll=Gtk.ScrolledWindow()
	sort=Gtk.TreeModelSort.new_with_model(list)
	t=show_univ(nb,scroll,sort,clkrow)
	b=Gtk.Button.new_with_label(chr(0x24D8))
	b.connect('clicked', info, [t,nb,win])
	page.append(scroll)
	page.append(b)
	return page
def show_univ(nb,sc,srt,cl_rw):
	sc.set_vexpand(True)
	t=hubs.TreeView(srt)
	renderer = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn()
	column.set_title("Name")
	column.pack_start(renderer,True)
	column.add_attribute(renderer, "text", 0)
	t.append_column(column, clk, srt)
	t.connect("row-activated",cl_rw,nb)
	t.set_activate_on_single_click(True)
	sc.set_child(t)
	return t
def clk(b,d):
	hubs.clk_univ(d,0)
def clkrow(t,p,c,b):
	m=t.get_model()
	user=m.get_value(m.get_iter(p),0)
	adr=b.get_tab_label_text(page)
	ldload(adr,user)

def ldload(adr,user):
	reqs.requ("list.download",{"huburl" : adr, "nick" : user})

def clear(nb,adr):
	list.clear()
	nb.set_tab_label_text(page,adr)
def ifclear(nb,a):
	if(nb.get_tab_label_text(page)==a):
		clear(nb,intro)

def set(nb,adr,lst):
	clear(nb,adr)
	for x in lst:
		overrides.append(list,[x])

def info(b,triple):
	s=triple[0].get_selection()
	model,iter=s.get_selected()
	if iter:
		userinfo.ini(triple[1].get_tab_label_text(page),model.get_value(iter,0),triple[2])




from gi.repository import Gtk

from . import sets

import subprocess

file=Gtk.EntryBuffer()#text=''
end=Gtk.EntryBuffer()#text=''
f=None

def confs():
	f=Gtk.Frame(label="Log file")
	g=Gtk.Grid()
	lb=Gtk.Label(halign=Gtk.Align.START,label="Location")
	g.attach(lb,0,0,1,1)
	en=sets.entries(file)
	g.attach(en,1,0,1,1)
	lb=Gtk.Label(halign=Gtk.Align.START,label="External command when closing")
	g.attach(lb,0,1,1,1)
	en=sets.entries(end)
	g.attach(en,1,1,1,1)
	f.set_child(g)
	return f
def store(d):
	d['log_file']=finish()
	d['log_end']=end.get_text()
def restore(d):
	file.set_text(d['log_file'],-1)
	end.set_text(d['log_end'],-1)

def ini():
	log=file.get_text()
	if len(log)>0:
		global f
		f=open(log,"w")

def add(obj):
	if f:
		f.write(obj.__str__()+"\n")
		f.flush()

def finish():
	global f
	if f:
		txt=end.get_text()
		if txt:
			z=subprocess.run(txt,capture_output=True)
			f.write(str(z.stdout))
		f.close()
		f=None
	return file.get_text()

def reset():
	d=finish()
	if len(d)>0:
		global f
		f=open(d,"a")



from gi.repository import Gtk,GLib,GObject

from . import flist
from . import reqs
from . import hubs
from . import dload
from . import details
from . import extension
from . import sets
from . import overrides

from enum import IntEnum
class COLUMNS(flist.COLUMNS,IntEnum):
	USERS=len(flist.COLUMNS)
	FUSERS=USERS+1
	DETAIL=FUSERS+1
lastcolumn=len(flist.COLUMNS)+len(COLUMNS)

list=eval("Gtk.ListStore("+flist.listcols+",int,int,GObject.TYPE_PYOBJECT,GObject.TYPE_BOOLEAN)")
filter=list.filter_new()
sort=Gtk.TreeModelSort.new_with_model(filter)
page=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
info=Gtk.Label()
timer=0
limit=Gtk.EntryBuffer(text="50")
flag=False
extensions=Gtk.EntryBuffer()#text=""
restime=Gtk.EntryBuffer(text="10")
infs=Gtk.Label()
ext_fn="Filename"

def show():
	scroll=Gtk.ScrolledWindow()
	scroll.set_vexpand(True)
	tree=hubs.TreeView(sort)
	flist.cols(tree,clk)
	hubs.col(tree,'Users',COLUMNS.USERS,clk)
	hubs.col(tree,'Users with Free Slots',COLUMNS.FUSERS,clk)
	filter.set_visible_column(lastcolumn)
	sort.set_sort_column_id(COLUMNS.USERS,Gtk.SortType.DESCENDING)
	tree.connect("row-activated",clkrow,sort)
	tree.set_activate_on_single_click(True)
	scroll.set_child(tree)
	page.append(info)
	page.append(scroll)
	page.append(infs)
	return page
def clk(b,ix):
	hubs.clk_univ(sort,ix)
	if len(filter)>0:
		resort()
def clkrow(tree,path,column,model):
	it=model.get_iter(path)
	dload.add(model,it)
	details.set(model.get_value(it,COLUMNS.DETAIL))

def reset():
	reshow()
	relimiting()
def resort():
	reshow()
	limiting()
def relimiting():
	a=len(filter)
	if a>0:
		limiting()
	resinf(a)
def reshow():
	for x in list:
		nm=list.get_value(x.iter,flist.COLUMNS.NAME)
		list.set_value(x.iter,lastcolumn,extension_filter(nm))#to be in filter,then in sort
def start(t):
	reqs.requ("search.send",{"searchstring":t})
def send(e,nb):
	t=e.get_text()
	start(t)
	info.set_text(t)
	nr=nb.page_num(page)
	if nb.get_current_page()!=nr:
		global flag
		flag=True
		nb.set_current_page(nr)
	close()
	timer=GLib.timeout_add_seconds(int(restime.get_text()),get,None)
def close():
	if timer:
		GLib.source_remove(timer)

def append(r):
	fr=r["Free Slots"]
	for d in list:
		if d[COLUMNS.TTH]==r["TTH"]:
			list.set_value(d.iter,COLUMNS.USERS,d[COLUMNS.USERS]+1)
			if fr!="0":
				list.set_value(d.iter,COLUMNS.FUSERS,d[COLUMNS.FUSERS]+1)
			details.update(r,fr,list,d.iter,COLUMNS.DETAIL)
			return
	nm=r[ext_fn]
	overrides.append(list,[nm,int(r["Real Size"]),r["TTH"],1,0 if fr=="0" else 1,[details.create(r,fr)],extension_filter(nm)])#need to be visible at sort for limit
def set():
	global flag
	if flag:
		flag=False
	else:
		setcomplex()
def setcomplex():
	list.clear()
	result=reqs.reque("search.getresults",{"huburl":''})#not send final results
	if result:
		for r in result:
			if "TTH" in r:#not working with Directory ,yet
				append(r)
		relimiting()
	else:
		resinf(0)
def get(d):
	setcomplex()
	info.set_text('')
	timer=0
	return False
def limiting():
	n=sort.iter_n_children(None)
	m=n-1
	n-=int(limit.get_text())
	for i in range(0,n):
		i1=sort.iter_nth_child(None,m-i)
		i2=sort.convert_iter_to_child_iter(i1)
		i3=filter.convert_iter_to_child_iter(i2)
		list.set_value(i3,lastcolumn,False)
def extension_filter(nm):
	ext=extensions.get_text()
	if ext:
		p=nm.rfind('.')
		if p==-1:
			return False
		ex=nm[p+1:]
		e=ext.split(";")
		for x in e:
			if x==ex:
				return True
		return False
	return True

def store(d):
	d['search_limit']=int(limit.get_text())
	extension.store(d)
	d['search_time']=int(restime.get_text())
def restore(d):
	limit.set_text(str(d['search_limit']),-1)
	extension.restore(d)
	restime.set_text(str(d['search_time']),-1)
def confs(win):
	f=Gtk.Frame(label="Search options")
	g=Gtk.Grid()
	lb=Gtk.Label(halign=Gtk.Align.START,label="Rows limit")
	g.attach(lb,0,0,1,1)
	en=sets.entries(limit)
	g.attach(en,1,0,1,1)
	lb=Gtk.Label(halign=Gtk.Align.START,label="Extensions (e1;e2...eN or unfiltered(blank))")
	g.attach(lb,0,1,1,1)
	en=sets.entries(extensions)
	g.attach(extension.confs(en,win),1,1,1,1)
	lb=Gtk.Label(halign=Gtk.Align.START,label="Show results after N seconds")
	g.attach(lb,0,2,1,1)
	en=sets.entries(restime)
	g.attach(en,1,2,1,1)
	f.set_child(g)
	return f

def resinf(a):
	b=len(list)
	c=len(sort)
	infs.set_text('Visible: '+str(c)+'; Hidden: filter '+str(b-a)+', limit '+str(a-c))




from gi.repository import Gtk

import xml.etree.ElementTree as ET
import urllib.request
import math

listdef=lambda:Gtk.ListStore(str,int,str,str)

from . import base
from . import hubscon
from . import sets
from . import overrides
from . import hublist

addr=Gtk.EntryBuffer(text='https://www.te-home.net/?do=hublist&get=hublist.xml')
file=Gtk.EntryBuffer()
lim=Gtk.EntryBuffer(text='200')

list=listdef()
sort=Gtk.TreeModelSort.new_with_model(list)

from enum import IntEnum
class COLUMNS(IntEnum):
	ADDRESS=0
	USERS=1
	COUNTRY=2
	SHARED=3
def treedef(lst,act,clkrow,data):
	tree=TreeView(lst)
	col(tree,'Address',COLUMNS.ADDRESS,act)
	col(tree,'Users',COLUMNS.USERS,act)
	col(tree,'Country',COLUMNS.COUNTRY,act)
	col(tree,'Shared',COLUMNS.SHARED,act)
	tree.connect("row-activated",clkrow,data)
	tree.set_activate_on_single_click(True)
	return tree
def col(tr,tx,ix,act):
	renderer = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn()
	column.set_title(tx)
	column.set_resizable(True)
	column.pack_start(renderer,True)
	column.add_attribute(renderer, "text", ix)
	tr.append_column(column,act,ix)

class TreeView(Gtk.TreeView):
	def __init__(self,model):
		Gtk.TreeView.__init__(self)
		self.set_model(model)
		#self.set_headers_clickable(True)is default
	def append_column(self,col,fn,ix):
		col.connect('clicked',fn,ix)
		col.set_clickable(True)
		Gtk.TreeView.append_column(self,col)

def confs():
	f=Gtk.Frame(label="Hub List")
	g=Gtk.Grid()
	lb=Gtk.Label(halign=Gtk.Align.START,label="File address")
	g.attach(lb,0,0,1,1)
	en=sets.entries(addr)
	g.attach(en,1,0,1,1)
	lb=Gtk.Label(halign=Gtk.Align.START,label="File fallback location")
	g.attach(lb,0,1,1,1)
	g.attach(sets.entries(file),1,1,1,1)
	lb=Gtk.Label(halign=Gtk.Align.START,label="Maximum number of entries")
	g.attach(lb,0,2,1,1)
	en=sets.entries(lim)
	g.attach(en,1,2,1,1)
	f.set_child(g)
	return f
def store(d):
	d['hub_file']=addr.get_text()
	d['hub_file_fallback']=file.get_text()
	d['hub_limit']=lim.get_text()
def restore(d):
	addr.set_text(d['hub_file'],-1)
	file.set_text(d['hub_file_fallback'],-1)
	lim.set_text(d['hub_limit'],-1)

def reset():
	list.clear()
	ini()

def clk_univ(lst,ix):
	n=lst.get_sort_column_id()
	if n[1]!=Gtk.SortType.ASCENDING:
		lst.set_sort_column_id(ix,Gtk.SortType.ASCENDING)
	else:
		lst.set_sort_column_id(ix,Gtk.SortType.DESCENDING)
def clk(b,ix):
	clk_univ(sort,ix)
def show():
	wn=Gtk.ScrolledWindow()
	wn.set_vexpand(True)
	tree=treedef(sort,clk,hubscon.add,sort)
	wn.set_child(tree)
	return wn

def ini():
	try:
		tree = ET.ElementTree(file=urllib.request.urlopen(addr.get_text()))
		root = tree.getroot()
	except Exception:
		if file.get_text():
			tree = ET.parse(file.get_text())
			root = tree.getroot()
		else:
			root = ET.fromstring(hublist.a)
	try:
		hbs=root.find("Hubs").findall("Hub")
	except Exception:
		return
	mx=min(int(lim.get_text()),len(hbs))
	for i in range(mx):
		attrs=hbs[i].attrib
		if ('Secure' in attrs) and (attrs['Secure']):
			huburl=attrs['Secure']
		elif 'Address' in attrs:
			huburl=attrs['Address']
		else:
			continue
		if ('Users' in attrs) and ('Country' in attrs) and ('Shared' in attrs):
			overrides.append(list,[huburl,int(attrs['Users']),attrs['Country'],convert_size(int(attrs['Shared']))])

def convert_size(size_bytes):
	if size_bytes == 0:
		return "0 B"
	size_name = ("B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB")
	i = math.floor(math.log(size_bytes, 1024))
	p = pow(1024, i)
	s = math.floor(size_bytes / p * 100)/100 # trunc for positiv/negativ
	return "%s %s" % (s, size_name[i])



from gi.repository import Gtk,Gdk,GLib

import shlex

from . import reqs
from . import sets

mems=[]
pos=0
text=Gtk.TextView(editable=False,wrap_mode=Gtk.WrapMode.WORD_CHAR)
#CheckButton is Bus error here
verbose=Gtk.EntryBuffer()

def show():
	bx=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	scroll=Gtk.ScrolledWindow(vexpand=True,child=text)
	bx.append(scroll)
	global entry
	entry=Gtk.Entry(hexpand=True,placeholder_text='Command Name0 Value0 ... NameN ValueN')
	global controller
	controller=Gtk.EventControllerKey()
	controller.connect("key-pressed",eve,entry)
	entry.add_controller(controller)
	entry.connect("activate",send,None)
	bx.append(entry)
	return bx
def send(e,d):
	t=e.get_text()
	mem(t)
	n=shlex.split(t)
	c=n[0]
	del n[0]
	d={}
	m=len(n)&(-2)
	for i in range(0,m,2):
		d[n[i]]=n[i+1]
	r=reqs.reque_simple(c,d)
	if not 'result' in r:
		r='Error'
	addtext(r)
	e.get_buffer().delete_text(0,-1)
def addtext(r):
	b=text.get_buffer()
	it=b.get_end_iter()
	b.insert(it,str(r)+'\r\n',-1)
	GLib.idle_add(delay,text)
def delay(t):#this will be after row wrap
	b=t.get_buffer()
	it=b.get_end_iter()
	t.scroll_to_iter(it,0,False,0,0)
	return False
def mem(s):
	if len(mems)>4:
		del mems[0]
	mems.append(s)
	global pos
	pos=len(mems)
def eve(controller,keyval,keycode,state,entry):
	global pos
	if keyval==Gdk.KEY_Up:
		if pos>0:
			pos-=1
			entry.get_buffer().set_text(mems[pos],-1)
			return True
	elif keyval==Gdk.KEY_Down:
		x=len(mems)
		if pos<x:
			pos+=1
			if pos==x:
				entry.get_buffer().delete_text(0,-1)
				return True
			else:
				entry.get_buffer().set_text(mems[pos],-1)
				return True
	return False
def close():
	entry.remove_controller(controller)

def post(r):
	if verbose.get_text():
		addtext(r)

def confs():
	return sets.entry("Verbose (blank=False)",verbose)
def store(d):
	d['verbose']='1' if verbose.get_text() else ''
def restore(d):
	verbose.set_text(d['verbose'],-1)



from gi.repository import Gtk

from . import hubs
from . import hubscon
from . import users
from . import reqs
from . import overrides

list=hubs.listdef()
sort=Gtk.TreeModelSort.new_with_model(list)
page=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

def clk(b,ix):
	hubs.clk_univ(sort,ix)
def show(nb):
	wn=Gtk.ScrolledWindow()
	wn.set_vexpand(True)
	t=hubs.treedef(sort,clk,rowclk,nb)
	wn.set_child(t)
	page.append(wn)
	b=Gtk.Button.new_with_label("-")
	b.connect('clicked', rem, [t,nb])
	page.append(b)
	return page

def add(a):
	overrides.append(list,a)
def rem(b,gr):
	s=gr[0].get_selection()
	d=s.get_selected()#iter free is in the bindings
	if d[1]:
		hubscon.remcon(gr[1],d[0].get_value(d[1],hubs.COLUMNS.ADDRESS))
		list.remove(d[0].convert_iter_to_child_iter(d[1]))

def rowclk(tree,path,column,nb):
	model=tree.get_model()
	it=model.get_iter(path)
	adr=model.get_value(it,hubs.COLUMNS.ADDRESS)
	resp=reqs.reque("hub.getusers",{"huburl" : adr})
	users.set(nb,adr,resp.split(';'))

def store(d):
	l=[]
	for r in list:
		v=[]
		for i in hubs.COLUMNS:
			v.append(list.get_value(r.iter,i))
		l.append(v)
	d['hubs']=l
def restore(d):
	l=d['hubs']
	for r in l:
		hubscon.addcon(list,r)

def set():
	res=reqs.req("hub.listfulldesc")
	if res:
		for z in list:
			x=list.get_value(z.iter,hubs.COLUMNS.ADDRESS)
			for r in res:
				if r==x:
					list.set_value(z.iter,hubs.COLUMNS.USERS,int(res[r]["users"]))
					list.set_value(z.iter,hubs.COLUMNS.SHARED,res[r]["totalshare preformatted"])
					del res[r]
					break



from gi.repository import Gtk

import xml.etree.ElementTree as ET
import appdirs
import os.path

from . import limit
from . import nick
from . import sets

file=Gtk.EntryBuffer()
set='Settings'

def get_file_default():
	return os.path.join(appdirs.user_config_dir('eiskaltdc++'),'DCPlusPlus.xml')
def get_file():
	a=file.get_text()
	if a:
		return os.path.expandvars(a)
	return get_file_default()

def ini():
	f = get_file()
	t = ET.parse(f)
	root = t.getroot()
	s = root.find(set)
	limit.start=int(s.find('TotalUpload').text)
def confs():
	f=Gtk.Frame(label="External data file settings")
	g=Gtk.Grid()
	lb=Gtk.Label(halign=Gtk.Align.START,label="Location (blank for default)")
	g.attach(lb,0,0,1,1)
	g.attach(confs_loc(),1,0,1,1)
	lb=Gtk.Label(halign=Gtk.Align.START,label="Nick name")
	g.attach(lb,0,1,1,1)
	en=sets.entries(nick.confs())
	g.attach(en,1,1,1,1)
	f.set_child(g)
	return f
def store(d):
	d['ext_file']=file.get_text()
def restore(d):
	file.set_text(d['ext_file'],-1)

def confs_loc():
	en=sets.entries(file)
	if file.get_text():
		return en
	bx=Gtk.Box()
	bx.append(en)
	bx.append(Gtk.Label(label=get_file_default()))
	return bx